package com.gic.cloud.web.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.dto.AttentionStoreDTO;
import com.gic.cloud.dto.DataAuthDTO;
import com.gic.cloud.dto.TempStoreConditionDTO;
import com.gic.cloud.service.DataAuthApiService;
import com.gic.cloud.service.StoreAttentionApiService;
import com.gic.cloud.service.TempStoreConditionApiService;
import com.gic.cloud.web.auth.DataAuthUtils;
import com.gic.cloud.web.auth.OnLineAuth;
import com.gic.cloud.web.auth.OnLineStore;
import com.gic.cloud.web.auth.StoreAuth;
import com.gic.cloud.web.constant.Constants;
import com.gic.cloud.web.constant.StoreChannelEnum;
import com.gic.cloud.web.qo.StoreSearchQo;
import com.gic.cloud.web.vo.LevelVo;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.constant.StoreOwnerFlagEnum;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreGroupDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreGroupApiService;
import com.gic.store.service.StoreWidgetApiService;
import com.gic.weimob.api.dto.WeimobStoreGicInfoDTO;
import com.gic.weimob.api.service.WeimobStoreSiteApiService;
import com.google.inject.internal.cglib.core.$ClassEmitter;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.reflections.Store;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
/**
 * 门店查询工具
 * @ClassName: StoreSearchUtils 
 * @Description:  
 * @author boom 
 * @date 2020/9/7 1:34 PM 
 */
@Component
public class StoreSearchUtils {
    private static final Logger log = LogManager.getLogger(StoreSearchUtils.class);
    @Autowired
    private DataAuthUtils dataAuthUtils;
    @Autowired
    private StoreAttentionApiService storeAttentionApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreGroupApiService storeGroupApiService;
    @Autowired
    private DataAuthApiService dataAuthApiService;
    @Autowired
    private WeimobStoreSiteApiService weimobStoreSiteApiService;
    @Autowired
    private WmStoreApiService wmStoreApiService;

    public Integer isWgs(Integer userId, Integer enterpriseId, String searchJson){
        if(StringUtils.isBlank(searchJson)){
            return 0;
        }
        JSONObject json = JSON.parseObject(searchJson);
        StoreSearchQo storeSearchQo = json.getObject(StoreChannelEnum.OFFLINE.getChannel().toString(), StoreSearchQo.class);
        log.info("searchJSON:{},storeSearchQo:{}", searchJson, JSON.toJSONString(storeSearchQo));
        if(storeSearchQo != null){
            return storeSearchQo.getHasWgs();
        }
        return 0;
    }

    public int dealSearchJson(String searchJson){
        /***线上和线下选项值都为空的时候--所有门店权限范围
         * 线上或者线下有一个的选项值不为空时，则为单渠道权限范围*
         * 线上和线下的json值为空的时候 -- 所有门店权限范围*/
        if(StringUtils.isBlank(searchJson)){
            return 0;
        } else {
            JSONObject json = JSON.parseObject(searchJson);
            StoreSearchQo offLineSearch = json.getObject(StoreChannelEnum.OFFLINE.getChannel().toString(), StoreSearchQo.class);
            StoreSearchQo onLineSearch = JSON.parseObject(searchJson).getObject(StoreChannelEnum.WMMALL.getChannel().toString(), StoreSearchQo.class);
            boolean off = offLineSearch == null || offLineSearch.isEmpty();
            boolean online = onLineSearch == null || onLineSearch.isEmpty();
            if(off && online){
                return 0;
            } else if(!off && online){
                return 1;
            } else if(off && !online){
                return 2;
            } else {
                return 0;
            }
        }
    }

    public List<Integer> storeSearch(Integer userId, Integer enterpriseId, String searchJson){
        log.info("storeSearchParams:{},{},{}", userId, enterpriseId, searchJson);
        List<Integer> resultList = new ArrayList<>();
        int flag = dealSearchJson(searchJson);

        boolean isQueryOnline = flag == 2;
        if(isQueryOnline){
            resultList.add(0);
            return resultList;
        }
        String key = "1:" + enterpriseId + ":" + userId +":" + (StringUtils.isNotBlank(searchJson) ? searchJson.hashCode() : "searchJSON".hashCode());
        StoreAuth storeAuth = this.dataAuthUtils.getStoreAuth(userId, enterpriseId);
        log.info("storeAuth:{}", JSON.toJSONString(storeAuth));
        if(!storeAuth.isHasAuth()){
            resultList.add(0);
            return resultList;
        }
        if(StringUtils.isNotBlank(searchJson)){
            JSONObject json = JSON.parseObject(searchJson);
            StoreSearchQo storeSearchQo = json.getObject(StoreChannelEnum.OFFLINE.getChannel().toString(), StoreSearchQo.class);
            StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
            if(storeSearchQo.getAll() == 1){
                storeSearchDTO.setStoreInfoIds(StringUtils.join(storeAuth.getStoreInfoIdList().toArray(), " "));
            }else {
                if (StringUtils.isNotBlank(storeSearchQo.getStoreIds())) {
                    storeSearchDTO.setStoreInfoIds(storeSearchQo.getStoreIds().replaceAll(",", " "));
                }
            }
            storeSearchDTO.setEnterpriseId(enterpriseId);
            storeSearchDTO.setStoreGroupIds(StringUtils.isBlank(storeSearchQo.getStoreGroupIds()) ? null : storeSearchQo.getStoreGroupIds().replaceAll(",",  " "));
            storeSearchDTO.setRegionIds(StringUtils.isBlank(storeSearchQo.getRegion()) ? null : storeSearchQo.getRegion().replaceAll(",",  " "));
            storeSearchDTO.setStoreTypes(StringUtils.isBlank(storeSearchQo.getStoreType()) ? null :storeSearchQo.getStoreType().replaceAll(",",  " "));
            storeSearchDTO.setStoreStatuss(StringUtils.isBlank(storeSearchQo.getStoreStatus()) ? null :storeSearchQo.getStoreStatus().replaceAll(",",  " "));
            storeSearchDTO.setRealStatuss(StringUtils.isBlank(storeSearchQo.getErpStatus()) ? null :storeSearchQo.getErpStatus().replaceAll(",",  " "));
            storeSearchDTO.setStoreTags(StringUtils.isBlank(storeSearchQo.getStoreTag()) ? null :storeSearchQo.getStoreTag().replaceAll(",",  " "));
            storeSearchDTO.setStoreBrandIds(StringUtils.isBlank(storeSearchQo.getStoreBrand()) ? null :storeSearchQo.getStoreBrand().replaceAll(",",  " "));
            String resourceType = storeSearchQo.getResourceType();
            Integer ownerFlag = null;
            if(StringUtils.isNotBlank(resourceType)){
                boolean isOwner = "1".equals(resourceType);
                if(isOwner){
                    ownerFlag = StoreOwnerFlagEnum.OWNER.getCode();
                };;
                boolean isNotOwner = "2".equals(resourceType);
                if(isNotOwner){
                    ownerFlag = StoreOwnerFlagEnum.OTHER.getCode();
                }
            }
            if(storeSearchQo.getAttentionStore() == 1){
                List<AttentionStoreDTO> result = this.storeAttentionApiService.pageStoreAttention(userId, enterpriseId, 1, Integer.MAX_VALUE).getResult().getResult();
                List<Integer> list = result.stream().filter(t -> {
                    if(t.getStoreId() != null){
                        return true;
                    }else {
                        return false;
                    }
                }).map(dto -> dto.getStoreId()).collect(Collectors.toList());
                String storeInfoIds = StringUtils.join(list, " ");
                storeSearchDTO.setStoreInfoIds(StringUtils.isNotBlank(storeSearchDTO.getStoreInfoIds()) ? storeSearchDTO.getStoreInfoIds()+" "+ storeInfoIds : storeInfoIds);
            }
            ServiceResponse<Page<StoreDTO>> pageServiceResponse = this.storeApiService.listStore(storeSearchDTO, 1, 20000);
            if(pageServiceResponse.isSuccess() && pageServiceResponse.getResult() != null && CollectionUtils.isNotEmpty(pageServiceResponse.getResult().getResult())){
                List<StoreDTO> storeDTOList = pageServiceResponse.getResult().getResult();
                Integer finalOwnerFlag = ownerFlag;
                List<Integer> storeInfoIdList = storeDTOList.stream().filter(storeDTO -> {
                    if(finalOwnerFlag != null){ //按资源类型进行过滤
                        if(storeDTO.getOwnerFlag().equals(finalOwnerFlag)){
                            return true;
                        }else {
                            return  false;
                        }
                    }
                    return true;
                }).map(storeDTO -> storeDTO.getStoreInfoId()).collect(Collectors.toList());
                if(CollectionUtils.isNotEmpty(storeAuth.getStoreInfoIdList())){
                    resultList = storeInfoIdList.stream().filter(s -> {
                        for (Integer storeInfoId : storeAuth.getStoreInfoIdList()) {  //按权限门店进行过滤
                            if (s != null && s.intValue() == storeInfoId.intValue()) {
                                return true;
                            }
                        }
                        return false;
                    }).collect(Collectors.toList());
                } else {
                    resultList = storeInfoIdList;
                }
            }
        } else {
            RedisUtil.setCache(key, 1, 10L, TimeUnit.MINUTES);
            return storeAuth.getStoreInfoIdList();
        }
        if(CollectionUtils.isNotEmpty(resultList)){
            RedisUtil.setCache(key, 1, 10L, TimeUnit.MINUTES);
        }else {
            resultList.add(0);
        }
        log.info("storeSearchResult:{}", JSON.toJSONString(resultList));
        return resultList;
    }

    public List<String> onLineStoreSearch(Integer userId, Integer enterpriseId, String searchJson){
        List<String> list = new ArrayList<>();
        int flag = dealSearchJson(searchJson);
        if(flag == 1){
            list.add("noauth");
            return list;
        }
        OnLineAuth onlineStoreAuth = this.dataAuthUtils.getOnlineStore(userId, enterpriseId);
        log.info("onlineStoreAuth:{}", JSON.toJSONString(onlineStoreAuth));
        if(onlineStoreAuth.isHasAuth()){
            this.getOnlineStoreIds(searchJson, onlineStoreAuth, list, StoreChannelEnum.GICMALL.getChannel(), userId, enterpriseId);
            this.getOnlineStoreIds(searchJson, onlineStoreAuth, list, StoreChannelEnum.WMMALL.getChannel(), userId, enterpriseId);
            this.getOnlineStoreIds(searchJson, onlineStoreAuth, list, StoreChannelEnum.TIANMAO.getChannel(), userId, enterpriseId);
        }else {
            list.add("noauth");
        }
        return list;
    }

    public LevelVo getLevel(Integer userId, Integer enterpriseId, String searchJson){
        LevelVo vo = new LevelVo();
        List<Integer> list = this.storeSearch(userId, enterpriseId, searchJson);
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setStoreInfoIds(StringUtils.join(list, " "));
        ServiceResponse<Page<StoreDTO>> response = this.storeApiService.listStore(storeSearchDTO, 1, 20000);
        if(!response.isSuccess() && response.getResult() == null){
            return vo;
        }
        List<StoreDTO> result = response.getResult().getResult();
        Map<Integer, Integer> map = new HashMap<>(16);
        result.forEach(storeDTO -> {
            List<Integer> storeGroupIdList = storeDTO.getStoreGroupIdList();
            if (CollectionUtils.isNotEmpty(storeGroupIdList)) {
                storeGroupIdList.forEach(storeGroupId -> {
                    map.put(storeGroupId, storeGroupId);
                });
            }
        });
        List<StoreGroupDTO> allStoreGroupIdList = this.storeGroupApiService.listStoreGroupByIds(null, enterpriseId).getResult();
        Map<Integer, Integer> levelMap = new HashMap<>(16);
        allStoreGroupIdList.forEach(storeGroupDTO -> {
            if(map.containsKey(storeGroupDTO.getStoreGroupId())){
                levelMap.put(storeGroupDTO.getGroupLevel(), levelMap.get(storeGroupDTO.getGroupLevel())== null ? 1 : levelMap.get(storeGroupDTO.getGroupLevel())+1);
            }
        });
        int start = 0;
        if(!levelMap.isEmpty()){
            while (true){
                if(start >= levelMap.size()){
                    break;
                }
                if(levelMap.get(start) != null && levelMap.get(start).intValue() > 1){
                    vo.setLevel(start);
                    break;
                }
                start++;
            }
            vo.setMaxLevel(levelMap.size() - 1);
        }
        return vo;
    }

    public Integer getChannel(Integer userId, Integer enterpriseId, String searchJson){
        String offlineKey = "1:" + enterpriseId + ":" + userId +":" + (StringUtils.isNotBlank(searchJson) ? searchJson.hashCode() : "searchJSON".hashCode());
        String gicMallKey = "2:" + enterpriseId + ":" + userId +":" + (StringUtils.isNotBlank(searchJson) ? searchJson.hashCode() : "searchJSON".hashCode());
        String wmKey = "3:" + enterpriseId + ":" + userId +":" + (StringUtils.isNotBlank(searchJson) ? searchJson.hashCode() : "searchJSON".hashCode());
        String tmKey = "4:" + enterpriseId + ":" + userId +":" + (StringUtils.isNotBlank(searchJson) ? searchJson.hashCode() : "searchJSON".hashCode());
        int offline = RedisUtil.getCache(offlineKey) == null ? 0 : 1;
        int gicMall = RedisUtil.getCache(gicMallKey) == null ? 0 : 1;
        int wm = RedisUtil.getCache(wmKey) == null ? 0 : 1;
        int tm = RedisUtil.getCache(tmKey) == null ? 0 : 1;
        if((offline + gicMall + wm + tm) > 1){
            //多渠道
            return Constants.ZERO;
        }else if((offline + gicMall + wm + tm)  == 1){
            if(offline == 1){
                return offline;
            }
            if(gicMall == 1){
                return gicMall;
            }
            if(wm == 1){
                return wm;
            }
            if(tm == 1){
                return tm;
            }
        }
        return null;
    }

    public List<Integer> getChildrenStoreGroupIdList(Integer enterpriseId, Integer parentId, Integer level, Integer channel){
        if(StoreChannelEnum.OFFLINE.getChannel().equals(channel)){
            int storeGroupLevel = 7;
            if(level < storeGroupLevel){
                if(parentId != null){
                    List<StoreGroupDTO> result = this.storeGroupApiService.listStoreGroupAndChildren(parentId).getResult();
                    if(CollectionUtils.isNotEmpty(result)){
                        return result.stream().map(storeGroupDTO -> storeGroupDTO.getStoreGroupId()).collect(Collectors.toList());
                    }
                }
            }else {
                StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
                storeSearchDTO.setEnterpriseId(enterpriseId);
                storeSearchDTO.setStoreGroupId(parentId);
                ServiceResponse<Page<StoreDTO>> response = this.storeApiService.listStore(storeSearchDTO, 1, 10000);
                if(response.isSuccess() && response.getResult() != null){
                    return response.getResult().getResult().stream().map(storeDTO -> storeDTO.getStoreInfoId()).collect(Collectors.toList());
                }
            }
        }
        if(StoreChannelEnum.WMMALL.getChannel().equals(channel)){
            List<WeimobStoreGicInfoDTO> list = weimobStoreSiteApiService.listStoreId(parentId).getResult();
            List<Integer> storeInfoIdList = list.stream().map(s -> s.getStoreInfoId()).collect(Collectors.toList());
            return storeInfoIdList;
        }

        return new ArrayList<>();
    }

    private void getOnlineStoreIds(String searchJson, OnLineAuth onlineStoreAuth, List<String> list, Integer channel, Integer userId, Integer enterpriseId){
        String key = channel + ":" + enterpriseId +":" + userId +":" + (StringUtils.isNotBlank(searchJson) ? searchJson.hashCode() : "searchJSON".hashCode());
        if(StringUtils.isNotBlank(searchJson)){
            StoreSearchQo searchQo = JSON.parseObject(searchJson).getObject(channel.toString(), StoreSearchQo.class);
            log.info("getOnlineStoreIds:{},{}", searchJson, JSON.toJSONString(searchQo));
            if(searchQo != null){
                if(searchQo.getAll() != 1){ //不是所有线上门店
                    if(StringUtils.isNotBlank(searchQo.getStoreGroupIds())){
                        List<String> filter = filter(channel, searchQo, enterpriseId, key, Arrays.asList(searchQo.getStoreGroupIds().split(",")));
                        list.addAll(filter);
                    }
                } else {
                    onlineStoreAuth.getList().stream().forEach(s -> {
                        if(s.getChannel() != null && s.getChannel().intValue() == channel.intValue()){
                            if(CollectionUtils.isNotEmpty(s.getStoreIdList())){
                                RedisUtil.setCache(key, 1, 10L, TimeUnit.MINUTES);
                                List<String> filter = filter(channel, searchQo, enterpriseId, key, s.getStoreIdList());
                                list.addAll(filter);
                            }
                        }
                    });
                }
            }
        } else {
            if(CollectionUtils.isNotEmpty(onlineStoreAuth.getList())){
                onlineStoreAuth.getList().stream().forEach(s -> {
                    if(s.getChannel() != null && s.getChannel().intValue() == channel.intValue()){
                        if(CollectionUtils.isNotEmpty(s.getStoreIdList())){
                            RedisUtil.setCache(key, 1, 10L, TimeUnit.MINUTES);
                            list.addAll(s.getStoreIdList());
                        }
                    }
                });
            }
        }
    }

    private List<String> filter(Integer channel, StoreSearchQo searchQo, Integer enterpriseId, String key, List<String> onLineStoreIdList) {
        if (StoreChannelEnum.WMMALL.getChannel().equals(channel)) {
            List<WmStoreDTO> wmStoreDTOList = this.wmStoreApiService.listWmStore(onLineStoreIdList, null).getResult();
            List<String> onLineStoreList = wmStoreDTOList.stream().filter(wmStoreDTO -> {
                boolean resourceFlag = true;
                if (StringUtils.isNotBlank(searchQo.getResourceType())) {
                    //自有资源
                    boolean isOwner = "1".equals(searchQo.getResourceType());
                    if (isOwner) {
                        if (enterpriseId.equals(wmStoreDTO.getEnterpriseId())) {
                            resourceFlag = true;
                        }
                    }
                    //共享资源
                    boolean isNotOwner = "2".equals(searchQo.getStoreBrand());
                    if (isNotOwner) {
                        if (!enterpriseId.equals(wmStoreDTO.getEnterpriseId())) {
                            resourceFlag = true;
                        }
                    }
                }
                boolean brandFlag = true;
                if (StringUtils.isNotBlank(searchQo.getStoreBrand())) {
                    List<String> strings = Arrays.asList(searchQo.getStoreBrand().split(","));
                    if (strings.contains(wmStoreDTO.getStoreBrandId().toString())) {
                        brandFlag = true;
                    }
                }
                return resourceFlag || brandFlag;
            }).map(wmStoreDTO -> wmStoreDTO.getWmMallStoreId().toString()).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(onLineStoreList)) {
                RedisUtil.setCache(key, 1, 10L, TimeUnit.MINUTES);
            }
            return onLineStoreList;
        }
        return null;
    }

}
